﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Telerik.Windows.DragDrop;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.GridView;

namespace WpfApplication8
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			this.DataContext = new MainViewModel();

			DragDropManager.AddDragInitializeHandler(SourceGrid, new DragInitializeEventHandler(OnDragInitialize));
			DragDropManager.AddDragDropCompletedHandler(SourceGrid, new DragDropCompletedEventHandler(OnDragDropComplete));

			this.SourceGrid.DataLoaded += new EventHandler<EventArgs>(SourceGrid_DataLoaded);

			DragDropManager.AddDragOverHandler(DestinationGrid, new Telerik.Windows.DragDrop.DragEventHandler(OnDragOver));
			DragDropManager.AddDragOverHandler(SourceGrid, new Telerik.Windows.DragDrop.DragEventHandler(OnSourceGridDragOver));

			DragDropManager.AddDragLeaveHandler(DestinationGrid, new Telerik.Windows.DragDrop.DragEventHandler(OnDragLeave));
			DragDropManager.AddDragLeaveHandler(SourceGrid, new Telerik.Windows.DragDrop.DragEventHandler(OnSourceGridDragLeave));


			this.dropPositionFeedbackPresenter = new ContentPresenter();
		}

		void SourceGrid_DataLoaded(object sender, EventArgs e)
		{
			this.AttachDropPositionFeedback();
		}

		public void OnDragDropComplete(object sender, DragDropCompletedEventArgs args)
		{
			this.HideDropPositionFeedbackPresenter();
		}

		public void OnDragInitialize(object sender, DragInitializeEventArgs args)
		{
			var data = ((FrameworkElement)args.Source).DataContext;
			args.Data = args.Source;
			args.DragVisual = new TreeListViewDragCue() { ItemsSource = new List<object> { data }, DragTooltipVisibility = System.Windows.Visibility.Collapsed };
			args.DragVisualOffset = new Point(args.RelativeStartPoint.X - 50, args.RelativeStartPoint.Y - 10);
			args.AllowedEffects = DragDropEffects.Link | DragDropEffects.Move;
		}


		public void OnDragOver(object sender, Telerik.Windows.DragDrop.DragEventArgs args)
		{
			if (args.OriginalSource != null && args.OriginalSource is UIElement)
			{
				var row = ((UIElement)args.OriginalSource).ParentOfType<GridViewRow>();

				row.IsSelected = true;

			}
		}

		public void OnSourceGridDragLeave(object sender, Telerik.Windows.DragDrop.DragEventArgs args)
		{
			this.HideDropPositionFeedbackPresenter();
		}

		public void OnSourceGridDragOver(object sender, Telerik.Windows.DragDrop.DragEventArgs args)
		{
			if (args.OriginalSource != null && args.OriginalSource is UIElement)
			{
				var row = ((UIElement)args.OriginalSource).ParentOfType<GridViewRow>();

				var currentDropPosition = this.GetDropPositionFromPoint(args.GetPosition(Window.GetWindow(row)), row);

				if ((args.AllowedEffects & DragDropEffects.Move) == DragDropEffects.Move || (args.AllowedEffects & DragDropEffects.Link) == DragDropEffects.Link)
				{
					this.ShowDropPositionFeedbackPresenter(sender as RadGridView, row, currentDropPosition);
				}
				else
				{
					this.HideDropPositionFeedbackPresenter();
				}
			}
			else
			{
				this.HideDropPositionFeedbackPresenter();
			}



		}

		public void OnDragLeave(object sender, Telerik.Windows.DragDrop.DragEventArgs args)
		{
			if (args.OriginalSource != null && args.OriginalSource is UIElement)
			{
				var row = ((UIElement)args.OriginalSource).ParentOfType<GridViewRow>();

				row.IsSelected = false;
			}

		}


		private static UIElement CreateDefaultDropPositionFeedback()
		{
			Grid grid = new Grid()
			{
				Height = 8,
				HorizontalAlignment = HorizontalAlignment.Stretch,
				IsHitTestVisible = false,
				VerticalAlignment = VerticalAlignment.Stretch
			};
			grid.ColumnDefinitions.Add(new ColumnDefinition()
			{
				Width = new GridLength(8)
			});
			grid.ColumnDefinitions.Add(new ColumnDefinition());
			Ellipse ellipse = new Ellipse()
			{
				Stroke = new SolidColorBrush(Colors.Orange),
				StrokeThickness = 2,
				Fill = new SolidColorBrush(Colors.Orange),
				HorizontalAlignment = HorizontalAlignment.Stretch,
				VerticalAlignment = VerticalAlignment.Stretch,
				Width = 8,
				Height = 8
			};
			Rectangle rectangle = new Rectangle()
			{
				Fill = new SolidColorBrush(Colors.Orange),
				RadiusX = 2,
				RadiusY = 2,
				VerticalAlignment = VerticalAlignment.Stretch,
				HorizontalAlignment = HorizontalAlignment.Stretch,
				Height = 2
			};
			Grid.SetColumn(ellipse, 0);
			Grid.SetColumn(rectangle, 1);
			grid.Children.Add(ellipse);
			grid.Children.Add(rectangle);
			return grid;
		}

		private Grid dropPositionFeedbackPresenterHost;

		//private void DetachDropPositionFeedback()
		//{
		//    if (this.IsDropPositionFeedbackAvailable())
		//        this.dropPositionFeedbackPresenterHost.Children.Remove(this.dropPositionFeedbackPresenter);
		//}

		private ContentPresenter dropPositionFeedbackPresenter;

		private void AttachDropPositionFeedback()
		{
			this.dropPositionFeedbackPresenterHost = SourceGrid.ChildrenOfType<Grid>().FirstOrDefault();
			if (dropPositionFeedbackPresenterHost != null)
			{
				dropPositionFeedbackPresenter.Content = CreateDefaultDropPositionFeedback();
				dropPositionFeedbackPresenterHost.Children.Add(this.dropPositionFeedbackPresenter);
			}
			this.HideDropPositionFeedbackPresenter();
		}

		public virtual DropPosition GetDropPositionFromPoint(Point absoluteMousePosition, GridViewRow row)
		{
			if (row != null)
			{
				var headerTopPoint = row.TransformToVisual(Window.GetWindow(row)).Transform(new Point());
				double mouseTop = absoluteMousePosition.Y - headerTopPoint.Y;
				if (mouseTop <= (row.ActualHeight * 1 / 2))
				{
					return DropPosition.Before;
				}
				else if (mouseTop > (row.ActualHeight * 1 / 2))
				{
					return DropPosition.After;
				}
			}
			return DropPosition.Before;
		}

		private void ShowDropPositionFeedbackPresenter(GridViewDataControl gridView, GridViewRow row, DropPosition lastRowDropPosition)
		{
			if (!this.IsDropPositionFeedbackAvailable())
				return;
			var yOffset = this.GetDropPositionFeedbackOffset(row, lastRowDropPosition);
			this.dropPositionFeedbackPresenter.Visibility = Visibility.Visible;
			this.dropPositionFeedbackPresenter.Width = row.ActualWidth;
			this.dropPositionFeedbackPresenter.RenderTransform = new TranslateTransform()
			{
				Y = yOffset
			};
		}

		private bool IsDropPositionFeedbackAvailable()
		{
			return
				this.dropPositionFeedbackPresenterHost != null &&
				this.dropPositionFeedbackPresenter != null;
		}

		private double GetDropPositionFeedbackOffset(GridViewRow row, DropPosition dropPosition)
		{
			var yOffset = row.TransformToVisual(this.dropPositionFeedbackPresenterHost).Transform(new Point(0, 0)).Y;
			if (dropPosition == DropPosition.After)
				yOffset += row.ActualHeight;
			yOffset -= (this.dropPositionFeedbackPresenter.ActualHeight / 2.0);
			return yOffset;
		}

		private void HideDropPositionFeedbackPresenter()
		{
			this.dropPositionFeedbackPresenter.RenderTransform = new TranslateTransform()
			{
				X = 0,
				Y = -234324
			};
		}
	}


	public class User : INotifyPropertyChanged
	{
		private int id;

		public int Id
		{
			get
			{
				return id;
			}
			set
			{
				id = value;
				this.OnPropertyChanged("Id");
			}
		}

		private string userName;

		public string UserName
		{
			get
			{
				return userName;
			}
			set
			{
				userName = value;
				this.OnPropertyChanged("UserName");
			}
		}

		private DateTime lastLoginDate;

		public DateTime LastLoginDate
		{
			get
			{
				return lastLoginDate;
			}
			set
			{
				lastLoginDate = value;
				this.OnPropertyChanged("LastLoginDate");
			}
		}


		private bool active;

		public bool Active
		{
			get
			{
				return active;
			}
			set
			{
				active = value;
				this.OnPropertyChanged("Active");
			}
		}


		private bool approved;

		public bool Approved
		{
			get
			{
				return approved;
			}
			set
			{
				approved = value;
				this.OnPropertyChanged("Approved");
			}
		}


		public event PropertyChangedEventHandler PropertyChanged;

		protected virtual void OnPropertyChanged(string propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}
	}

	public class MainViewModel
	{
		private ObservableCollection<User> data;

		public ObservableCollection<User> Data
		{
			get { return data; }
			set { data = value; }
		}

		private ObservableCollection<User> destinationData;

		public ObservableCollection<User> DestinationData
		{
			get { return destinationData; }
			set { destinationData = value; }
		}

		public MainViewModel()
		{
			GenerateData(10);

			this.destinationData = new ObservableCollection<User>((
				from p in Enumerable.Range(0, 10)
				select new User
				{
					Id = p,
					Active = p % 2 == 0,
					Approved = p % 3 == 0,
					LastLoginDate = DateTime.Now.AddDays(-p),
					UserName = "Destination User " + p
				}).ToList());
		}

		private void GenerateData(int c)
		{
			this.data = new ObservableCollection<User>((
				from p in Enumerable.Range(0, c)
				select new User
				{
					Id = p,
					Active = p % 2 == 0,
					Approved = p % 3 == 0,
					LastLoginDate = DateTime.Now.AddDays(-p),
					UserName = "User " + p
				}).ToList());
		}

	}

}
